-- $Rev: 72 $
WMS_016 = {}
local WMS = WMS_016

WMS.MSG_CODES = {
  REQUEST_TO_SEND = "RTS",
  CLEAR_TO_SEND = "CTS",
  TRANSMIT = "TX",
  RECEIVE = "RX",
  ACKNOWLEDGE = "ACK",
  BUFFER_TRANSFER = "TRANS",
  NOTIFY_REDIRECTED = "REDIR"
}

function WMS:SetReceivedText(text)
    self.receivedText = text
    self:GetReceivedFontString():SetText(text)
end

function WMS:GetReceivedText()
    if (not self.receivedText) then
        self.receivedText = ""
    end
    return self.receivedText
end

function WMS:SetLastReceivedMessage(message, sender)
    self.lastReceivedMessage = {
        message = message,
        sender = sender
    }
end

function WMS:GetLastReceivedMessage()
    return self.lastReceivedMessage
end

function WMS:ApplySendMsgBtnDesaturation()
    local sendIcon = self:GetChildWidget("SendMsgButtonIconTexture")
    SetDesaturation(sendIcon, (string.len(self:GetEditText()) == 0) or not self:GetReceiver())
end

function WMS:SetEditText(text, dontUpdateWidget)
    local oldValid = self.editText and (string.len(self.editText) > 0)
    self.editText = text
    local valid = self.editText and (string.len(self.editText) > 0)
    
    self:ApplySendMsgBtnDesaturation()
    
    if (valid and (not oldValid) and self:GetReceiver()) then
        self:StartMsgButtonAnimIn()
    end
    
    if (not dontUpdateWidget) then
        self:GetMessageEditBox():SetText(text)
    end
end

function WMS:GetEditText()
    if (not self.editText) then
        self:SetEditText("")
    end
    return self.editText
end

function WMS:SetBeamedOnBy(newBeamedOnBy)
  self.beamedOnBy = newBeamedOnBy
  
  local texturePath
  if (self.beamedOnBy) then
    texturePath = "Interface\\AddOns\\WMS-016\\WMS_RecEnable"
    PlaySoundFile("Sound\\DOODAD\\LampostCreek3.ogg", "sfx")
  else
    texturePath = "Interface\\AddOns\\WMS-016\\WMS_OFF"
  end
  
  local tex = self:GetRecEnableTexture()
  tex:SetTexture(texturePath)
--   print("SetBeamedOnBy beamedOnBy:", self:GetBeamedOnBy())
end

function WMS:GetBeamedOnBy()
  return self.beamedOnBy
end

function WMS:SetReceiver(newReceiver)
    local oldReceiver = self.receiver
    self.receiver = newReceiver
    if (newReceiver and string.len(self:GetEditText()) > 0) then
        if (not oldReceiver) then
            self:StartMsgButtonAnimIn()
        end
    end

    self:ApplySendMsgBtnDesaturation()
end

function WMS:GetReceiver()
  return self.receiver 
end

function WMS:IsSendEnabled()
  return self:GetReceiver() ~= nil
end

function WMS:GetSavedVariables()
  if (not self.SavedVariables) then
    if (not WMS_016_SavedVariables) then
      WMS_016_SavedVariables = {}
      --print("new WMS_016_SavedVariables", WMS_016_SavedVariables)
    end
  
    self.SavedVariables = WMS_016_SavedVariables
    --print("self.SavedVariables assigned", self.SavedVariables)
  end
  return self.SavedVariables
end

function WMS:GetConCount(counterType)
  local sv = self:GetSavedVariables()
  if (not sv.conCount) then
    sv.conCount = {}
  end
  
  if (not sv.conCount[counterType]) then
    sv.conCount[counterType] = 0
  end
  
  return sv.conCount[counterType]
end

function WMS:ResetConCount()
    local sv = self:GetSavedVariables()
    sv.conCount = {}
    self:UpdateConCount()
end

function WMS:IncConCount(counterType)
  local count = self:GetConCount(counterType)
  count = count + 1
  
  local cc = self:GetSavedVariables().conCount
  cc[counterType] = count
  
  self:CheckForUpgrade(counterType)
  self:UpdateConCount()
end


------------------------------------------------------------------------------

function WMS:OnBeamOnChanged(beamIsOn, caster, beamReceiver)
  local myName = UnitName("PLAYER")
--~   print ("beamIsOn:", beamIsOn,
--~     " Caster:", caster,
--~     " beamReceiver:", beamReceiver,
--~     " \"PLAYER\":", myName)
    
  if (myName == caster) then
    self.mainFrame:Show()
    self:OnBeamCastChanged(beamIsOn, beamReceiver)
  end
  
  if (myName == beamReceiver) then
    self.mainFrame:Show()
    self:SetBeamedOnBy(beamIsOn and caster or nil)
  end
  
end

-- I cast a beam. That generated this event to myself.
function WMS:OnBeamCastChanged(beamIsOn, beamReceiver) 
  --print("OnBeamSent beamReceiver:", beamReceiver)
  if (beamIsOn) then
    self:SetReceiver(beamReceiver)
    PlaySoundFile("Sound\\DOODAD\\ZulDrak_Crystal_01_ClosedOneShot03.ogg", "sfx")
    self.COM:SendMsg(self.MSG_CODES.REQUEST_TO_SEND, "", beamReceiver)
    self:StartNeedleBeamDuration()
  else
    self:SetReceiver(nil)
    local tex = WMS:GetSendEnableTexture()
    tex:SetTexture("Interface\\AddOns\\WMS-016\\WMS_OFF")
    self:StopNeedleBeamDuration()
  end
end


-- The BeamSender (not me, I'm the BeamReceiver) requests my permission to allow him to send me a message.
function WMS:OnRequestToSend(message)
  --print("RTS received") 
  self:IncConCount(self.MSG_CODES.REQUEST_TO_SEND)
  
  self.mainFrame:Show()
  
  -- Answer the BeamSenders request with a permission "CTS" 
  self.COM:SendMsg(self.MSG_CODES.CLEAR_TO_SEND, "", self:GetBeamedOnBy())
end

-- The BeamReceiver (not me, I'm the BeamSender) signaled to me the permisson to send him messages
function WMS:OnClearToSend(message, sender)
  --print("CTS received from:", sender)
  self:IncConCount(self.MSG_CODES.CLEAR_TO_SEND)
  
  local tex = WMS:GetSendEnableTexture()
  tex:SetTexture("Interface\\AddOns\\WMS-016\\WMS_SendEnable")
  PlaySoundFile("Sound\\DOODAD\\LampostCreek3.ogg", "sfx")
end

function WMS:OnTx(message, sender)
  self:IncConCount(self.MSG_CODES.RECEIVE)

  if (self.ReceivedMsgCorrosionTimer 
    and not WMS.ReceivedMsgCorrosionTimer:IsTimedOut()) 
  then
    self.ReceivedMsgCorrosionTimer:Stop()
  end
 
  self:SetReceivedText("")
  self:StartReceiveAnim(message)
  self:SetLastReceivedMessage(message, sender)
  self.COM:SendMsg(self.MSG_CODES.ACKNOWLEDGE, "", self:GetBeamedOnBy())
  self:UpdateConCount()
  
  self.ReceivedMsgCorrosionTimer = self.CreateTimer(20, 
    function ()
      WMS:SetReceivedText("")
      PlaySound("igMiniMapClose", "sfx")
    end
  )
  self.ReceivedMsgCorrosionTimer:Start()
  self.corroAnim:Stop()
  self.corroAnim:Play()
  self.cogwheelLeftAnimBwd:Play()    
  self.cogwheelRightAnimBwd:Play()    

end

function WMS:OnAck()
  self:StartSendAnim(self:GetEditText())
  self.cogwheelLeftAnimFwd:Play()
  self.cogwheelRightAnimFwd:Play()
  self:IncConCount(self.MSG_CODES.ACKNOWLEDGE)
end


function WMS:OnRedirectedNotification(message, sender)
    self:RedirectionNotified(message, sender)
end

function WMS:OnMessage(prefix, body, sender)
  local MessageHandler = {
    [self.MSG_CODES.REQUEST_TO_SEND] = WMS.OnRequestToSend ,
    [self.MSG_CODES.CLEAR_TO_SEND] = WMS.OnClearToSend,
    [self.MSG_CODES.TRANSMIT] = WMS.OnTx,
    [self.MSG_CODES.ACKNOWLEDGE] = WMS.OnAck,
    [self.MSG_CODES.NOTIFY_REDIRECTED] = WMS.OnRedirectedNotification,
  }
  local selectedHandler = MessageHandler[prefix]
  if (selectedHandler) then
    selectedHandler(self, body, sender)
  end
end

function WMS:OnMessageSend()
  local msg = self:GetEditText()
  if (self:IsSendEnabled() and msg and msg ~= "") then
    self.COM:SendMsg(self.MSG_CODES.TRANSMIT, msg, self:GetReceiver())
    self:IncConCount(self.MSG_CODES.TRANSMIT)
    self:RedirectedCheck(self:GetLastReceivedMessage(), msg)
  else
    self.cogwheelLeftAnimStuck:Play()
    self.cogwheelRightAnimFwd:Play()    
  end
end

function WMS:OnBufferTransfer()
    if (string.len(self:GetReceivedText()) > 0) then
        self:IncConCount(self.MSG_CODES.BUFFER_TRANSFER)
    end
    WMS:StartBufferTransferAnim()
end

function WMS:OnCrypt()
    local recTxt = self:GetReceivedText()
    local eb = self:GetMessageEditBox()
    local edTxt = self:GetEditText()
    
    self:StartCryptionAnim(recTxt..edTxt)
    
    self:GetReceivedFontString():Hide()
    eb:Hide()
    
    local timer = WMS.CreateTimer(0.8, 
        function()
            recTxt = self:Crypt(recTxt)
            edTxt = self:Crypt(edTxt)
            
            self:SetReceivedText(recTxt)
            self:SetEditText(edTxt)
            
            self:GetReceivedFontString():Show()
            eb:Show()
        end
    )
    timer:Start()
end


function WMS:ShakeText(text)
    local dest = ""
    while #text >= 1 do   
        local letterIdx = math.random (1, #text)
        local selLetter = string.sub(text, letterIdx, letterIdx)
        dest = dest .. selLetter
        text = string.sub(text, 1, letterIdx - 1) .. string.sub(text, letterIdx + 1) or ""
    end
    return dest
end
--------------------------------------------------------------------------
function WMS:PrintDebug(...)
    if (self.debug) then
        print(...)
    end
end