-- $Rev: 25 $
local WMS = WMS_016
local LF = WMS.letterFactory

function WMS:InitCogwheelAnim()
    self.cogwheelLeftAnimFwd = self:GetChildWidget("CogwheelLeftAnimFwd")
       
    if (self.cogwheelLeftAnimFwd) then
        self.cogwheelLeftAnimFwd:SetScript(
            "OnPlay", 
            function ()
                PlaySoundFile("Interface\\AddOns\\WMS-016\\WMS_Cogwheel.ogg", "sfx")
            end
        )
    else
        print("Fehler am linken Zahnrad")
    end

    self.cogwheelLeftAnimBwd = self:GetChildWidget("CogwheelLeftAnimBwd")
       
    if (self.cogwheelLeftAnimBwd) then
        self.cogwheelLeftAnimBwd:SetScript(
            "OnPlay", 
            function ()
                PlaySoundFile("Interface\\AddOns\\WMS-016\\WMS_Cogwheel.ogg", "sfx")
            end
        )
    else
        print("Fehler am linken Zahnrad")
    end
    
    self.cogwheelLeftAnimStuck = self:GetChildWidget("CogwheelLeftAnimStuck")
    if (self.cogwheelLeftAnimStuck) then
        self.cogwheelLeftAnimStuck:SetScript(
            "OnPlay", 
            function ()
                PlaySoundFile("Interface\\AddOns\\WMS-016\\WMS_Cogwheel_krk.ogg", "sfx")
            end
        )
        
        local prevX, prevY
        for animIdx = 0, 10 do            
            local trans = self.cogwheelLeftAnimStuck:CreateAnimation("Translation", "anim"..tostring(animIdx))
            if (animIdx % 2 == 0) then
                prevX = -10 + math.random(5) 
                prevY = -10 + math.random(5) 
                trans:SetOffset(prevX, prevY) 
            else
                trans:SetOffset(-prevX, -prevY) 
            end
            trans:SetDuration(0.1)
            trans:SetOrder(animIdx)
        end
    else
        print("Fehler am linken Zahnrad")
    end
    
    self.cogwheelRightAnimFwd = self:GetChildWidget("CogwheelRightAnimFwd")
    self.cogwheelRightAnimBwd = self:GetChildWidget("CogwheelRightAnimBwd")
end

function WMS:PlayLetterTransferSound()
    local glassTimer = self.CreateTimer(0.75, 
        function()
            PlaySoundFile("Interface\\AddOns\\WMS-016\\WMS_Letters_glass.ogg", "sfx")
        end
    )

    local boxTimer = self.CreateTimer(1.5, 
        function()
            PlaySoundFile("Interface\\AddOns\\WMS-016\\WMS_Letters_box.ogg", "sfx")
        end
    )

    glassTimer:Start()
    boxTimer:Start()
end

function WMS:StartBufferTransferAnim()
    local success = false
    local letterPath = {
        {["x"]=0,   ["y"]=-65, ["width"]=115, ["height"]=55},
        {["x"]=102,  ["y"]=0,   ["width"]=5,   ["height"]=5},
        {["x"]=102,  ["y"]=40,  ["width"]=5,   ["height"]=5},
        {["x"]=102,  ["y"]=40,  ["width"]=128, ["height"]=5},
        {["x"]=224, ["y"]=40,  ["width"]=5,   ["height"]=5},
        {["x"]=224, ["y"]=0,   ["width"]=5,   ["height"]=5},
        {["x"]=195, ["y"]=-55, ["width"]=120, ["height"]=0},
    }
        
    local lf = self:GetLetterFactory("TransferAnim")
    lf:SetLetterParent(self:GetChildWidget("Received"))
    lf:SetLetterLayer("BACKGROUND")
    lf:SetOnPlayHandler(
        function ()
            local rTxt = WMS:GetReceivedText()
            WMS:SetReceivedText(string.sub(rTxt, 2))
            return string.sub(rTxt, 1, 1)
        end
    )
    lf:SetOnFinishedHandler(
        function(letterTxt)
            local ebTxt = WMS:GetEditText()
            ebTxt = ebTxt .. lf:GetCurrentLetter()
            WMS:SetEditText(ebTxt)
        end
    )
    lf:SetLetterPath(letterPath)
    lf:SetLetterOffX(3)
    lf:SetLetterOffY(0)
    lf:SetVarX(5)
    lf:SetVarY(5)
    lf:SetDuration(2)
    lf:SetMaxDelay(0.7)
    lf:SetLooping("NONE")
    
    if (lf:GetLettersInAnimCount() == 0) then
        success = self:StartTransferEngineAnim()
        if (success) then            
            local recTxt = WMS:GetReceivedText()
            lf:PlayString(recTxt)       
            if (recTxt and #recTxt > 0) then
                self:PlayLetterTransferSound()
            end
        end
    end
    return success
end

function WMS:StartTransferEngineAnim()
    local success = true
    local animationNames = {   
        "SpringR",
        "SpringL",
        "PistonHeadL",
        "PistonHeadR",
        "ValveL",
        "ValveR"
    }
    
    for i, animName in ipairs(animationNames) do
        local anim = self:GetChildWidget("Engine"..animName.."Inhale")
        if anim:IsPlaying() then
            return false
        end
        anim:Play()
    end
    
    PlaySoundFile("Interface\\AddOns\\WMS-016\\WMS_TransferEngine.ogg", "sfx")
    return success
end

function WMS:StartReceiveAnim(msg)
    local letterPath = {
        {["x"]=0,   ["y"]=-265, ["width"]=0, ["height"]=0},
        {["x"]=60,  ["y"]=-280,   ["width"]=100,   ["height"]=40},
    }
    
    local lf = self:GetLetterFactory("Receive")
    lf:SetLetterParent(self.mainFrame)
    lf:SetLetterLayer("BACKGROUND")
    lf:SetOnPlayHandler(nil)
    lf:SetOnFinishedHandler(
        function(letterTxt)
            local recTxt = WMS:GetReceivedText()
            recTxt = recTxt .. lf:GetCurrentLetter()
            WMS:SetReceivedText(recTxt)
            
            if (lf:GetLettersInAnimCount() == 0) then
                if (WMS.isPlayerMoving) then
                    WMS:StartMovingShake()
                end
            end
        end
    )
    lf:SetLetterPath(letterPath)
    lf:SetLetterOffX(-20)
    lf:SetLetterOffY(150)
    lf:SetVarX(15)
    lf:SetVarY(15)
    lf:SetDuration(0.8)
    lf:SetMaxDelay(0.4)
    lf:SetLooping("NONE")
    lf:PlayString(msg)
    
    self.CreateTimer(.5, 
        function()
            PlaySoundFile("Interface\\AddOns\\WMS-016\\WMS_Letters_box.ogg", "sfx")
        end
    ):Start()

end

function WMS:StartSendAnim(msg)
    local letterPath = {
        {["x"]=115,  ["y"]=15,   ["width"]=0,  ["height"]=0},
        {["x"]=115,  ["y"]=50, ["width"]=10, ["height"]=10},
        {["x"]=40,  ["y"]=300, ["width"]=120, ["height"]=160},
    }

    local lf = self:GetLetterFactory("SendAnim")
    lf:SetLetterParent(self:GetMessageEditBox():GetParent())
    lf:SetLetterLayer("BACKGROUND")
    lf:SetOnPlayHandler(
        function()      
            local txt = WMS:GetEditText()
            local letter = string.sub(txt, 1, 1)
            txt = string.sub(txt, 2)
            WMS:SetEditText(txt)
            return letter
        end
    )
    lf:SetOnFinishedHandler(nil)
    lf:SetLetterPath(letterPath)
    lf:SetLetterOffX(60)
    lf:SetLetterOffY(-30)
    lf:SetVarX(10)
    lf:SetVarY(10)
    lf:SetDuration(0.8)
    lf:SetMaxDelay(0.4)
    lf:SetLooping("NONE")
    lf:PlayString(msg)
end

function WMS:SetLongNeedle(percent)
    local rot = WMS:GetChildWidget("NeedleLongAnimationRotation")
    local degree = (50 * percent) / 100 - 25
    rot:SetDegrees(degree)
    --print("charges :", percent," degree ", degree)

    local bd = WMS:GetChildWidget("NeedleLongAnimBeamDuration")
    if (not bd:IsPlaying()) then    
        local na = WMS:GetChildWidget("NeedleLongAnimation")
        if (not (na:IsPaused() or na:IsPlaying())) then
            na:Play()
            na:Pause()
        end
    end
end

function WMS:StartNeedleBeamDuration()
    local na = WMS:GetChildWidget("NeedleLongAnimation")
    na:Stop()
    
    local ba = WMS:GetChildWidget("NeedleLongAnimBeamDuration")
    ba:Play()
    
    local ns = WMS:GetChildWidget("NeedleLongShiver")
    ns:Play()
end

function WMS:StopNeedleBeamDuration()
    local ba = WMS:GetChildWidget("NeedleLongAnimBeamDuration")
    ba:Stop()
    
    local ns = WMS:GetChildWidget("NeedleLongShiver")
    ns:Stop()
end

function WMS:StartMovingShake()
    self:StartMovingShakeReceived()
    self:StartMovingShakeEdited()
end

function WMS:StartMovingShakeReceived()
    local letterPath = {
        {["x"]=0,   ["y"]=10, ["width"]=5, ["height"]=40}
    }
    
    local lf = self:GetLetterFactory("MovingShakeReceived")
    self.MovingShakeActive = true
    lf:SetLetterParent(self:GetChildWidget("Received"))
    lf:SetOnPlayHandler(nil)
    lf:SetOnFinishedHandler(nil)
    lf:SetOnLoop(
        function(animGrp, loopstate)
            if (not WMS.MovingShakeActive) then
                animGrp:Stop()
            end
        end
    )
    lf:SetLetterPath(letterPath)
    lf:SetLetterOffX(60)
    lf:SetLetterOffY(-53)
    lf:SetVarX(120)
    lf:SetVarY(0)
    lf:SetDuration(0.2)
    lf:SetMaxDelay(0.3)
    lf:SetLooping("repeat")
    lf:PlayString(self:GetReceivedText())
    
    self:GetReceivedFontString():Hide()
end

function WMS:StartMovingShakeEdited()
    local letterPath = {
        {["x"]=0,   ["y"]=10, ["width"]=5, ["height"]=40}
    }
    
    local lf = self:GetLetterFactory("MovingShakeEdited")
    self.MovingShakeActive = true
    lf:SetLetterParent(self:GetChildWidget("MessageScrollframe"))
    lf:SetOnPlayHandler(nil)
    lf:SetOnFinishedHandler(nil)
    lf:SetOnLoop(
        function(animGrp, loopstate)
            if (not WMS.MovingShakeActive) then
                animGrp:Stop()
            end
        end
    )
    lf:SetLetterPath(letterPath)
    lf:SetLetterOffX(60)
    lf:SetLetterOffY(-53)
    lf:SetVarX(120)
    lf:SetVarY(0)
    lf:SetDuration(0.2)
    lf:SetMaxDelay(0.3)
    lf:SetLooping("repeat")
    lf:PlayString(self:GetEditText())
    
    self:GetMessageEditBox():Hide()
end

function WMS:StopMovingShake()
   self:StopMovingShakeReceived()
   self:StopMovingShakeEdited()
end

function WMS:StopMovingShakeReceived()
    self.MovingShakeActive = false
   
    local text = self:GetReceivedText()
    text = self:ShakeText(text)
    
    self:SetReceivedText(text)
    self:GetReceivedFontString():Show()
end

function WMS:StopMovingShakeEdited()
    self.MovingShakeActive = false
   
    local text = self:GetEditText()
    text = self:ShakeText(text)
   
   self:SetEditText(text)
   self:GetMessageEditBox():Show()
end


function WMS:StartCryptionAnim(text)
    local letterPath = {
        {["x"]=0,   ["y"]=-35, ["width"]=10, ["height"]=10},
        {["x"]=-50,   ["y"]=-80, ["width"]=10, ["height"]=10},
        {["x"]=-100,   ["y"]=-35, ["width"]=10, ["height"]=10},
        {["x"]=-95,   ["y"]=-3, ["width"]=10, ["height"]=10},
    }
    
    local animParent = self:GetChildWidget("Crypter")
    
    local lf = self:GetLetterFactory("CryptionAnim")
    lf:SetLetterParent(animParent)
    lf:SetLetterLayer("ARTWORK")
    lf:SetOnLoop(nil)
    lf:SetLetterPath(letterPath)
    lf:SetLetterOffX(110)
    lf:SetLetterOffY(-5)
    lf:SetVarX(5)
    lf:SetVarY(5)
    lf:SetDuration(0.4)
    lf:SetMaxDelay(0.4)
    lf:SetLooping("NONE")
    lf:PlayString(text)
    
    if (text and string.len(text) > 0) then
        PlaySoundFile("Interface\\AddOns\\WMS-016\\WMS_Letters_glass.ogg", "sfx")
    else 
        PlaySound("igMiniMapClose", "sfx")
    end
    
end

function WMS:StartMsgButtonAnimIn()
    local animIn = self:GetChildWidget("SendMsgButtonAnimIn")
    animIn:Play()
end

function WMS:StartRedirectorTimeoutAnim(duration)
    local needleAnim = self:GetChildWidget("Redirector".."Needle".."AnimGrp")
    if duration then
        local needleRot2 = self:GetChildWidget("Redirector".."Needle".."AnimGrp" .. "Rot2")
        needleRot2:SetDuration(duration - 0.5) -- Rot1 takes 0.5 sec
    end
    needleAnim:Stop()
    needleAnim:Play()
end

function WMS:StartRedirectorFlashAnim()
    local flashAnim = self:GetChildWidget("Redirector".."FontString".."Flash")
    if (not flashAnim:IsPlaying()) then
        flashAnim:Play()
    end
end

function WMS:SetRedirectionTextTilt()
    local redirTiltAnim = WMS:GetChildWidget("Redirector".."FontString".."Tilt")
    if (not (redirTiltAnim:IsPaused() or redirTiltAnim:IsPlaying())) then        
        redirTiltAnim:Play()
        redirTiltAnim:Pause()
    end
end