-- $Rev: 72 $
local WMS = WMS_016
local PRISMATIC_LENS_ID = 112498
local SPELL_ID_PRISMATIC_FOCUS_LENS = 163441

local beamIsOn = nil

function WMS:ParseExecCombatLogEvent(msg, caster, victim, spellnr)
  if (victim and (victim ~= caster)) then
    if (spellnr == SPELL_ID_PRISMATIC_FOCUS_LENS) then
      --print("msg:", msg, " caster:", caster, " victim:", victim, " spellnr:", spellnr)
      
      if (msg == "SPELL_AURA_APPLIED") then
	beamIsOn = true
      elseif (msg == "SPELL_AURA_REMOVED") then
	beamIsOn = false
      end
      
      if (beamIsOn ~= nil) then
	WMS:OnBeamOnChanged(beamIsOn, caster, victim)
      end      
    end
  end
end

function WMS.OnCombatLogEventUnfiltered(frame, ...)
  local timestamp, combat_evt, hideCaster,
    sourceGUID, sourceName, sourceFlags, sourceRaidFlags,
    destGUID, destName, destFlags, destRaidFlags, spellnr = ...
  local destIsPlayer = (strfind(tostring(destGUID), "Player") == 1)
  if (destIsPlayer) then
    WMS:ParseExecCombatLogEvent(combat_evt, sourceName, destName, select(12, ...))
  end
  WMS:UpdateBeamButton()
end

function WMS:SetDynamicHandlersActive(active)
    for event in pairs(self:GetHandlersDynamic()) do
        if (active) 
        then self.mainFrame:RegisterEvent(event)
        else self.mainFrame:UnregisterEvent(event)
        end
    end
end

function WMS.OnPlayerRegenDisabled(frame, ...)
    WMS.mainFrame:Hide()
    WMS:SetDynamicHandlersActive(false)
end

function WMS.OnPlayerRegenEnabled(frame, ...)
    WMS:SetDynamicHandlersActive(true)
end

function WMS:GetHandlersPermanent()
    if (self.handlersPermanent == nil) then
        self.handlersPermanent = {
            ["PLAYER_REGEN_DISABLED"] = WMS.OnPlayerRegenDisabled,
            ["PLAYER_REGEN_ENABLED"] = WMS.OnPlayerRegenEnabled    
        }
    end
    return self.handlersPermanent
end

function WMS:GetHandlersDynamic()
    if (self.handlersDynamic == nil)then
        self.handlersDynamic = {
            ["COMBAT_LOG_EVENT_UNFILTERED"] = WMS.OnCombatLogEventUnfiltered,
            ["CHAT_MSG_ADDON"] = WMS.COM.OnRawMsg,
            ["BAG_UPDATE"] = WMS.UpdateBeamButton,
        }
    end
    return self.handlersDynamic
end

function WMS.PlayerStartedMoving()
    WMS.isPlayerMoving = true
    WMS:StartMovingShake()
end

function WMS.PlayerStoppedMoving()
    WMS:StopMovingShake()
    WMS.isPlayerMoving = false
end

WMS.wasMoving = false
WMS.wasFalling = false
function WMS.OnUpdate(frame, elapsed)
    local nowMoving = (GetUnitSpeed("PLAYER") > 0)
    if (WMS.wasMoving ~= nowMoving) then
        if (nowMoving) then
            WMS:PlayerStartedMoving()
        else
            WMS:PlayerStoppedMoving()
        end
        WMS.wasMoving = nowMoving
    end
    
    local nowFalling = IsFalling()
    if (nowFalling  ~= WMS.wasFalling) then
        if (nowFalling) then
            WMS:PlayerStartedMoving()
        else
            WMS:PlayerStoppedMoving()
        end
        WMS.wasFalling = nowFalling
    end
end

function WMS:GetHandlersAll()
    if (self.handlersAll == nil) then
        self.handlersAll = {
            WMS:GetHandlersPermanent(),
            WMS:GetHandlersDynamic()
        }
    end
    return self.handlersAll
end

function WMS.OnEvent (frame, event, ...)
    for hlk, handlerList in pairs(WMS:GetHandlersAll()) do
        local handler = handlerList[event]
        if (handler) then
            handler(frame, ...)
        end
    end
end

function WMS.OnLoad(frame, ...)
  WMS.mainFrame = frame
  
  WMS:CreateCom()
  
  WMS:InitCogwheelAnim()

  WMS.mainFrame:SetScript("OnEvent", WMS.OnEvent)
  WMS.mainFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
  WMS.mainFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
  WMS:SetDynamicHandlersActive(true)
  WMS:GetChildWidget("Redirector"):SetScript("OnShow",WMS.RedirectedOnShow)
end

function WMS:ApplyUpgrades()
    for _, upgrade in pairs(self.Upgrades) do
        upgrade:applyUpgrade()
    end
end

function WMS.OnReceiveDrag(frame)
    local typeInfo, itemId =  GetCursorInfo()
    if ("item" == typeInfo) then
        WMS:SetCrypterItem(itemId)
        ClearCursor()
    end
end

function WMS.Crypter_OnClick(button, mouseButton, down)
    WMS:OnCrypt()
end

local function SetDesaturation(texture, desaturation)
  local shaderSupported = texture:SetDesaturated(desaturation);
  if ( not shaderSupported ) then
    if ( desaturation ) then
      texture:SetVertexColor(0.5, 0.5, 0.5);
    else
      texture:SetVertexColor(1.0, 1.0, 1.0);
    end
  end
end

function WMS.SendMsgButton_OnClick(button, ...)
  WMS:OnMessageSend()
end

function WMS.BufferTransferButton_OnClick(self, ...)
    WMS:OnBufferTransfer()
end


function WMS.BeamButton_OnShow(beamButton)
  local sName, sLink, iRarity, iLevel, iMinLevel, 
        sType, sSubType, iStackCount, sEquipLoc, sTexture
        = GetItemInfo(PRISMATIC_LENS_ID)

  beamButton:SetAttribute("type", "item")
  beamButton:SetAttribute("item", sName)
  SetItemButtonTexture(beamButton, sTexture)
  beamButton:SetDisabledTexture("Interface\\AddOns\\WMS-016\\WMS_Disabled")
  WMS:UpdateBeamButton()
  --print("Setup Beambutton. Name:", sName, " Texture:", sTexture)
end

function WMS.MessageEdit_OnTextChanged(editBox, userInput)
    if userInput then
        WMS:SetEditText(editBox:GetText(), true)
    end
end

function WMS.UpdateBeamButton() 
    local charges = GetItemCount(PRISMATIC_LENS_ID, false, true)
    local btn = WMS:GetChildWidget("BeamButton")
    if (charges > 0) then
        btn:Enable()
    else
        btn:Disable()
    end    

    if (charges > 100) then
        charges = 100
    end
    WMS:SetLongNeedle(charges)
end

function WMS:UpdateConCount()
    local counterGroups = {
        RxConCount = {self.MSG_CODES.CLEAR_TO_SEND, self.MSG_CODES.RECEIVE, self.MSG_CODES.BUFFER_TRANSFER},
        TxConCount = {self.MSG_CODES.REQUEST_TO_SEND, self.MSG_CODES.TRANSMIT, self.MSG_CODES.ACKNOWLEDGE},
    }

    for widgetName, counterGroup in pairs(counterGroups) do
        local buf = ""
        for _, counterType in ipairs(counterGroup) do
            buf = buf 
                .. self:GetConCount(counterType)
                .. "\n"
        end
        local counterFontString = self:GetChildWidget(widgetName)
        counterFontString:SetText(buf)
    end
end

function WMS:UpdateChildWidgets()
    self:UpdateConCount()
    self:ApplyUpgrades()
end

function WMS:GetChildWidget(childNameSuffix)
  return _G[self.mainFrame:GetName()..childNameSuffix]
end

function WMS:GetReceivedFontString()
  local recFrm = self:GetChildWidget("ReceivedFontString")
  return recFrm
end

function WMS:GetMessageEditBox()
  return self:GetChildWidget("MessageScrollframeMessageEdit")
end

function WMS:GetRecEnableTexture()
  return self:GetChildWidget("RecEnableTex")
end

function WMS:GetSendEnableTexture()
  return self:GetChildWidget("SendEnableTex")
end

function WMS:GetRedirectorFontString()
    return self:GetChildWidget("Redirector".."FontString");
end