-- $Rev: 25 $
local WMS = WMS_016
local letterFactoryBase = {}

-- The parent widget of the FontString to be generated
function letterFactoryBase:SetLetterParent(letterParent)
    self.letterParent = letterParent
end

function letterFactoryBase:SetLetterLayer(layer)
    self.letterLayer = layer
end

--  Param onPlay is a function that returns a text. When the animation starts
-- 'onPlay' is called and the text is set in the FontString.
function letterFactoryBase:SetOnPlayHandler(onPlay)
    self.OnPlayHandler = onPlay
end

--  Param onFinished is a function that is called when the letter animation 
-- has finished. It is called with the text of the FontString as parameter
function letterFactoryBase:SetOnFinishedHandler(onFinished)
    self.OnFinishedHandler = onFinished
end

function letterFactoryBase:SetOnLoop(onLoop)
    self.OnLoopHandler = onLoop
end

function letterFactoryBase:SetLetterPath(letterPath)
    self.letterPath = letterPath
end

function letterFactoryBase:SetLetterOffX(offX)
    self.letterOffX = offX
end

function letterFactoryBase:SetLetterOffY(offY)
    self.letterOffY = offY
end

function letterFactoryBase:SetVarX(varX)    
    self.varX = varX
end

function letterFactoryBase:SetVarY(varY)    
    self.varY = varY
end

function letterFactoryBase:SetDuration(duration)
    self.duration = duration
end

function letterFactoryBase:SetMaxDelay(maxDelay)
    self.maxDelay = maxDelay
end

function letterFactoryBase:SetLooping(looping)
    self.looping = looping
end

function letterFactoryBase:SetLettersInAnimCount(counter)
    self.lettersInAnimCount = counter
end

function letterFactoryBase:GetLettersInAnimCount()
    if (not self.lettersInAnimCount) then
        self.lettersInAnimCount = 0
    end
    return self.lettersInAnimCount
end

function letterFactoryBase:GetCurrentLetter()
    local liac = self:GetLettersInAnimCount()
    local l_id = string.len(self.animatedString) - liac
    local c = string.sub(self.animatedString, l_id, l_id)
    return c
end

function letterFactoryBase:PlayString(forString)
    local success = false
    if (self:GetLettersInAnimCount() == 0) then
        self.animatedString = forString
        if forString then
            for i = 1, #forString do
                local fs = self:GetOrCreateLetter(i)
                if (fs) then
                    local letter = string.sub(forString, i, i)
                    fs:SetText(letter)
                    self:PlayLetter(fs)
                end
            end
        end
        success = true
    end
    return success
end

function letterFactoryBase:PlayLetter(fontString)
    for k, animGrp in pairs({fontString:GetAnimationGroups()}) do
        local delay = math.random() * self.maxDelay
        local timer = WMS.CreateTimer(delay, 
            function()
                animGrp:Play()
            end
        )
        timer:Start()
    end
end

function letterFactoryBase:GetOrCreateLetter(letterIndex)
    local fs = self:FindLetter(letterIndex)
    
    if (fs == nil) then
        fs = self:CreateLetter(letterIndex)
    end
        
    return fs
end

function letterFactoryBase.GetAnimationGrp(region, animationGrpName)
    local animGrps =  {region:GetAnimationGroups()}
    for i, animGrp in ipairs(animGrps) do
        if (animationGrpName == animGrp:GetName()) then
            return animGrp
        end
    end
end

function letterFactoryBase:FindLetter(letterIndex)
    local fs = nil
    
    for k, region in pairs(self.letters) do
        local regionName = region:GetName()
        if (region.letterIdx == letterIndex) then
            local pathGrp =  self.GetAnimationGrp(region, "PathAnimGrp")
            if (pathGrp and not pathGrp:IsPlaying()) then
                fs = region            
            end
            break
        end
    end
    return fs
end

function letterFactoryBase:CreateLetter(letterIndex)
    if (self.letterParent:IsShown()) then
        local layer = self.letterLayer or "BACKGROUND"
        local fs = self.letterParent:CreateFontString("AnimLetter", layer, "GameFontDisableSmall")
        table.insert(self.letters, fs)
        fs.letterIdx = letterIndex
--~         if (self.maxLetter or 0 <= letterIndex) then
--~             print("letter generated: ", letterIndex)
--~             self.maxLetter = letterIndex
--~         end

        local varX = math.random() * self.varX - (self.varX / 2)
        local varY = math.random() * self.varY - (self.varY / 2)
        fs:SetPoint("TOPLEFT", self.letterParent, "TOPLEFT", 
            self.letterOffX + varX,  self.letterOffY + varY)
        self:ApplyLetterAnimRotation(fs)
        self:ApplyLetterAnimPath(fs)
        return fs
    end
end

function letterFactoryBase:ApplyLetterAnimRotation(letterFontString)
    local fsAnimGrp = letterFontString:CreateAnimationGroup("letterRotation")
    fsAnimGrp:SetLooping("REPEAT")

    local animRot = fsAnimGrp:CreateAnimation("Rotation")
    animRot:SetDegrees(math.random(-359, 359))
    animRot:SetDuration(math.random()*4)
end
function letterFactoryBase:ToString()   
    return "LetterFactory " .. self.name
end

function letterFactoryBase:ApplyLetterAnimPath(letterFontString)
    local fsAnimGrp = letterFontString:CreateAnimationGroup("PathAnimGrp")
    fsAnimGrp:SetLooping(self.looping)
    fsAnimGrp:SetScript("OnLoop", self.OnLoopHandler)
    
    local animPath = fsAnimGrp:CreateAnimation("Path")   
    animPath:SetDuration(self.duration)
    animPath:SetCurve("SMOOTH")
    
    animPath.onPlayHook = self.OnPlayHandler
    animPath:SetScript("OnPlay", 
        function(ani)
            self:SetLettersInAnimCount(self:GetLettersInAnimCount() + 1)
            if (ani.onPlayHook) then                
                local txt = ani.onPlayHook()
                if (txt) then
                    letterFontString:SetText(txt)
                end
            end
            letterFontString:Show()
        end
    )
    
    animPath.onFinishedHook = self.OnFinishedHandler
    animPath:SetScript("OnFinished", 
        function(ani)
            self:SetLettersInAnimCount(self:GetLettersInAnimCount() - 1)
            letterFontString:Hide()
            if (ani.onFinishedHook) then
                local txt = letterFontString:GetText()
                ani.onFinishedHook(txt)
            end
        end
    )
        
    for i, rule in ipairs(self.letterPath) do
        local pt = animPath:CreateControlPoint()
        local x = rule.x + math.random(0, rule.width)
        local y = rule.y + math.random(0, rule.height)
        pt:SetOffset(x, y)
        pt:SetOrder(i)
    end
end

local function CreateLetterFactory()
    local letterFactory = 
    {
        letters = {}
    }
            
    setmetatable(letterFactory, {__index = letterFactoryBase})
    return letterFactory
end

local letterFactorys = {}
function WMS:GetLetterFactory(forName)
    local letterFactory = letterFactorys[forName]
    if (not letterFactory) then
        letterFactory = CreateLetterFactory()
        letterFactory.name = forName
        letterFactorys[forName] = letterFactory
    end
    return letterFactory
end